@@ -1,6 +1,6 @@
 // This module implements the portability layer for the Qt port of Scintilla.
 //
-// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -78,35 +78,6 @@ Point Point::FromLong(long lpoint)
 }
 
 
-// Colour palette management.  The Qt interface to colours means this class
-// doesn't have to do anything.
-Palette::Palette()
-{
-    used = 0;
-    allowRealization = false;
-}
-
-Palette::~Palette()
-{
-    Release();
-}
-
-void Palette::Release()
-{
-    used = 0;
-}
-
-void Palette::WantFind(ColourPair &cp, bool want)
-{
-    if (!want)
-        cp.allocated.Set(cp.desired.AsLong());
-}
-
-void Palette::Allocate(Window &)
-{
-}
-
-
 // Font management.
 Font::Font() : fid(0)
 {
@@ -116,8 +87,7 @@ Font::~Font()
 {
 }
 
-void Font::Create(const char *faceName, int, int size, bool bold, bool italic,
-        int flags)
+void Font::Create(const FontParameters &fp)
 {
     Release();
 
@@ -125,7 +95,7 @@ void Font::Create(const char *faceName, int, int size, bool bold, bool italic,
 
     QFont::StyleStrategy strategy;
 
-    switch (flags & SC_EFF_QUALITY_MASK)
+    switch (fp.extraFontFlag & SC_EFF_QUALITY_MASK)
     {
     case SC_EFF_QUALITY_NON_ANTIALIASED:
         strategy = QFont::NoAntialias;
@@ -150,16 +120,36 @@ void Font::Create(const char *faceName, int, int size, bool bold, bool italic,
     f->setStyleStrategy(strategy);
 
     // If name of the font begins with a '-', assume, that it is an XLFD.
-    if (faceName[0] == '-')
+    if (fp.faceName[0] == '-')
     {
-        f->setRawName(faceName);
+        f->setRawName(fp.faceName);
     }
     else
     {
-        f->setFamily(faceName);
-        f->setPointSize(size);
-        f->setBold(bold);
-        f->setItalic(italic);
+        f->setFamily(fp.faceName);
+        f->setPointSize(fp.size);
+
+        // See if the Qt weight has been passed via the back door.   Otherwise
+        // map Scintilla weights to Qt weights ensuring that the SC_WEIGHT_*
+        // values get mapped to the correct QFont::Weight values.
+        int qt_weight;
+
+        if (fp.weight < 0)
+            qt_weight = -fp.weight;
+        else if (fp.weight <= 200)
+            qt_weight = QFont::Light;
+        else if (fp.weight <= QsciScintillaBase::SC_WEIGHT_NORMAL)
+            qt_weight = QFont::Normal;
+        else if (fp.weight <= 600)
+            qt_weight = QFont::DemiBold;
+        else if (fp.weight <= 850)
+            qt_weight = QFont::Bold;
+        else
+            qt_weight = QFont::Black;
+
+        f->setWeight(qt_weight);
+
+        f->setItalic(fp.italic);
     }
 
     fid = f;
@@ -189,43 +179,43 @@ public:
 
     void Release();
     bool Initialised() {return painter;}
-    void PenColour(ColourAllocated fore);
+    void PenColour(ColourDesired fore);
     int LogPixelsY() {return 72;}
     int DeviceHeightFont(int points) {return points;}
     void MoveTo(int x_,int y_);
     void LineTo(int x_,int y_);
-    void Polygon(Point *pts, int npts, ColourAllocated fore,
-            ColourAllocated back);
-    void RectangleDraw(PRectangle rc, ColourAllocated fore,
-            ColourAllocated back);
-    void FillRectangle(PRectangle rc, ColourAllocated back);
+    void Polygon(Point *pts, int npts, ColourDesired fore,
+            ColourDesired back);
+    void RectangleDraw(PRectangle rc, ColourDesired fore,
+            ColourDesired back);
+    void FillRectangle(PRectangle rc, ColourDesired back);
     void FillRectangle(PRectangle rc, Surface &surfacePattern);
-    void RoundedRectangle(PRectangle rc, ColourAllocated fore,
-            ColourAllocated back);
-    void AlphaRectangle(PRectangle rc, int cornerSize, ColourAllocated fill,
-            int alphaFill, ColourAllocated outline, int alphaOutline,
+    void RoundedRectangle(PRectangle rc, ColourDesired fore,
+            ColourDesired back);
+    void AlphaRectangle(PRectangle rc, int cornerSize, ColourDesired fill,
+            int alphaFill, ColourDesired outline, int alphaOutline,
             int flags);
     void DrawRGBAImage(PRectangle rc, int width, int height, const unsigned char *pixelsImage);
-    void Ellipse(PRectangle rc, ColourAllocated fore, ColourAllocated back);
+    void Ellipse(PRectangle rc, ColourDesired fore, ColourDesired back);
     void Copy(PRectangle rc, Point from, Surface &surfaceSource);
 
-    void DrawTextNoClip(PRectangle rc, Font &font_, int ybase, const char *s,
-            int len, ColourAllocated fore, ColourAllocated back);
-    void DrawTextClipped(PRectangle rc, Font &font_, int ybase, const char *s,
-            int len, ColourAllocated fore, ColourAllocated back);
-    void DrawTextTransparent(PRectangle rc, Font &font_, int ybase,
-            const char *s, int len, ColourAllocated fore);
-    void MeasureWidths(Font &font_, const char *s, int len, int *positions);
-    int WidthText(Font &font_, const char *s, int len);
-    int WidthChar(Font &font_, char ch);
-    int Ascent(Font &font_);
-    int Descent(Font &font_);
-    int InternalLeading(Font &font_) {return 0;}
-    int ExternalLeading(Font &font_);
-    int Height(Font &font_);
-    int AverageCharWidth(Font &font_) {return WidthChar(font_, 'n');}
+    void DrawTextNoClip(PRectangle rc, Font &font_, XYPOSITION ybase,
+            const char *s, int len, ColourDesired fore, ColourDesired back);
+    void DrawTextClipped(PRectangle rc, Font &font_, XYPOSITION ybase,
+            const char *s, int len, ColourDesired fore, ColourDesired back);
+    void DrawTextTransparent(PRectangle rc, Font &font_, XYPOSITION ybase,
+            const char *s, int len, ColourDesired fore);
+    void MeasureWidths(Font &font_, const char *s, int len,
+            XYPOSITION *positions);
+    XYPOSITION WidthText(Font &font_, const char *s, int len);
+    XYPOSITION WidthChar(Font &font_, char ch);
+    XYPOSITION Ascent(Font &font_);
+    XYPOSITION Descent(Font &font_);
+    XYPOSITION InternalLeading(Font &font_) {return 0;}
+    XYPOSITION ExternalLeading(Font &font_);
+    XYPOSITION Height(Font &font_);
+    XYPOSITION AverageCharWidth(Font &font_);
 
-    int SetPalette(Palette *, bool) {return 0;}
     void SetClip(PRectangle rc);
     void FlushCachedState();
 
@@ -235,13 +225,14 @@ public:
     void DrawXPM(PRectangle rc, const XPM *xpm);
 
 private:
-    void drawText(PRectangle rc, Font &font_, int ybase, const char *s,
-            int len, ColourAllocated fore);
+    void drawRect(const PRectangle &rc);
+    void drawText(const PRectangle &rc, Font &font_, XYPOSITION ybase,
+            const char *s, int len, ColourDesired fore);
     QFontMetrics metrics(Font &font_);
     QString convertText(const char *s, int len);
-    static QColor convertQColor(const ColourAllocated &col,
+    static QColor convertQColor(const ColourDesired &col,
             unsigned alpha = 255);
-    static QRgb convertQRgb(const ColourAllocated &col, unsigned alpha);
+    static QRgb convertQRgb(const ColourDesired &col, unsigned alpha);
 
     bool unicodeMode;
     QPaintDevice *pd;
@@ -250,7 +241,7 @@ private:
     int pen_x, pen_y;
 };
 
-Surface *Surface::Allocate()
+Surface *Surface::Allocate(int)
 {
     return new SurfaceImpl;
 }
@@ -338,15 +329,15 @@ void SurfaceImpl::LineTo(int x_, int y_)
     pen_y = y_;
 }
 
-void SurfaceImpl::PenColour(ColourAllocated fore)
+void SurfaceImpl::PenColour(ColourDesired fore)
 {
     Q_ASSERT(painter);
 
     painter->setPen(convertQColor(fore));
 }
 
-void SurfaceImpl::Polygon(Point *pts, int npts, ColourAllocated fore,
-        ColourAllocated back)
+void SurfaceImpl::Polygon(Point *pts, int npts, ColourDesired fore,
+        ColourDesired back)
 {
     Q_ASSERT(painter);
 
@@ -360,23 +351,23 @@ void SurfaceImpl::Polygon(Point *pts, int npts, ColourAllocated fore,
     painter->drawPolygon(qpts);
 }
 
-void SurfaceImpl::RectangleDraw(PRectangle rc, ColourAllocated fore,
-        ColourAllocated back)
+void SurfaceImpl::RectangleDraw(PRectangle rc, ColourDesired fore,
+        ColourDesired back)
 {
     Q_ASSERT(painter);
 
     painter->setPen(convertQColor(fore));
     painter->setBrush(convertQColor(back));
-    painter->drawRect(rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top);
+    drawRect(rc);
 }
 
-void SurfaceImpl::FillRectangle(PRectangle rc, ColourAllocated back)
+void SurfaceImpl::FillRectangle(PRectangle rc, ColourDesired back)
 {
     Q_ASSERT(painter);
 
     painter->setPen(Qt::NoPen);
     painter->setBrush(convertQColor(back));
-    painter->drawRect(rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top);
+    drawRect(rc);
 }
 
 void SurfaceImpl::FillRectangle(PRectangle rc, Surface &surfacePattern)
@@ -392,15 +383,16 @@ void SurfaceImpl::FillRectangle(PRectangle rc, Surface &surfacePattern)
 
         painter->setPen(Qt::NoPen);
         painter->setBrush(brsh);
-        painter->drawRect(rc.left, rc.top, rc.right - rc.left,
-                rc.bottom - rc.top);
+        drawRect(rc);
     }
     else
-        FillRectangle(rc, ColourAllocated(0));
+    {
+        FillRectangle(rc, ColourDesired(0));
+    }
 }
 
-void SurfaceImpl::RoundedRectangle(PRectangle rc, ColourAllocated fore,
-        ColourAllocated back)
+void SurfaceImpl::RoundedRectangle(PRectangle rc, ColourDesired fore,
+        ColourDesired back)
 {
     Q_ASSERT(painter);
 
@@ -411,20 +403,20 @@ void SurfaceImpl::RoundedRectangle(PRectangle rc, ColourAllocated fore,
 }
 
 void SurfaceImpl::AlphaRectangle(PRectangle rc, int cornerSize,
-        ColourAllocated fill, int alphaFill, ColourAllocated outline,
+        ColourDesired fill, int alphaFill, ColourDesired outline,
         int alphaOutline, int)
 {
     Q_ASSERT(painter);
 
     int w = rc.right - rc.left;
     int h = rc.bottom - rc.top;
-
     QImage img(w, h, 32);
 
     img.fill(convertQRgb(fill, alphaFill));
     img.setAlphaBuffer(true);
 
-    // Assume that "cornerSize" means outline width.
+    // Assume that "cornerSize" means outline width (which it doesn't but keep
+    // this for historical reasons).
     if (cornerSize > 0)
     {
         QRgb oline = convertQRgb(outline, alphaOutline);
@@ -455,8 +447,13 @@ void SurfaceImpl::AlphaRectangle(PRectangle rc, int cornerSize,
     painter->drawImage(rc.left, rc.top, img);
 }
 
-void SurfaceImpl::Ellipse(PRectangle rc, ColourAllocated fore,
-        ColourAllocated back)
+void SurfaceImpl::drawRect(const PRectangle &rc)
+{
+    painter->drawRect(rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top);
+}
+
+void SurfaceImpl::Ellipse(PRectangle rc, ColourDesired fore,
+        ColourDesired back)
 {
     Q_ASSERT(painter);
 
@@ -485,8 +482,8 @@ void SurfaceImpl::Copy(PRectangle rc, Point from, Surface &surfaceSource)
     }
 }
 
-void SurfaceImpl::DrawTextNoClip(PRectangle rc, Font &font_, int ybase,
-        const char *s, int len,ColourAllocated fore, ColourAllocated back)
+void SurfaceImpl::DrawTextNoClip(PRectangle rc, Font &font_, XYPOSITION ybase,
+        const char *s, int len, ColourDesired fore, ColourDesired back)
 {
     Q_ASSERT(painter);
 
@@ -494,8 +491,8 @@ void SurfaceImpl::DrawTextNoClip(PRectangle rc, Font &font_, int ybase,
     drawText(rc, font_, ybase, s, len, fore);
 }
 
-void SurfaceImpl::DrawTextClipped(PRectangle rc, Font &font_, int ybase,
-        const char *s, int len, ColourAllocated fore, ColourAllocated back)
+void SurfaceImpl::DrawTextClipped(PRectangle rc, Font &font_, XYPOSITION ybase,
+        const char *s, int len, ColourDesired fore, ColourDesired back)
 {
     Q_ASSERT(painter);
 
@@ -504,8 +501,8 @@ void SurfaceImpl::DrawTextClipped(PRectangle rc, Font &font_, int ybase,
     painter->setClipping(false);
 }
 
-void SurfaceImpl::DrawTextTransparent(PRectangle rc, Font &font_, int ybase,
-        const char *s, int len, ColourAllocated fore)
+void SurfaceImpl::DrawTextTransparent(PRectangle rc, Font &font_,
+        XYPOSITION ybase, const char *s, int len, ColourDesired fore)
 {
     // Only draw if there is a non-space.
     for (int i = 0; i < len; ++i)
@@ -516,8 +513,8 @@ void SurfaceImpl::DrawTextTransparent(PRectangle rc, Font &font_, int ybase,
         }
 }
 
-void SurfaceImpl::drawText(PRectangle rc, Font &font_, int ybase,
-        const char *s, int len, ColourAllocated fore)
+void SurfaceImpl::drawText(const PRectangle &rc, Font &font_, XYPOSITION ybase,
+        const char *s, int len, ColourDesired fore)
 {
     QString qs = convertText(s, len);
 
@@ -534,11 +531,11 @@ void SurfaceImpl::DrawXPM(PRectangle rc, const XPM *xpm)
 {
     Q_ASSERT(painter);
 
-    int x, y;
+    XYPOSITION x, y;
     const QPixmap &qpm = xpm->Pixmap();
 
-    x = rc.left + (rc.Width() - qpm.width()) / 2;
-    y = rc.top + (rc.Height() - qpm.height()) / 2;
+    x = rc.left + (rc.Width() - qpm.width()) / 2.0;
+    y = rc.top + (rc.Height() - qpm.height()) / 2.0;
 
     painter->drawPixmap(x, y, qpm);
 }
@@ -548,17 +545,13 @@ void SurfaceImpl::DrawRGBAImage(PRectangle rc, int width, int height,
 {
     Q_ASSERT(painter);
 
-    int x, y;
     const QImage *qim = reinterpret_cast<const QImage *>(pixelsImage);
 
-    x = rc.left + (rc.Width() - width) / 2;
-    y = rc.top + (rc.Height() - height) / 2;
-
-    painter->drawImage(x, y, *qim);
+    painter->drawImage(rc.left, rc.top, *qim);
 }
 
 void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len,
-        int *positions)
+        XYPOSITION *positions)
 {
     QFontMetrics fm = metrics(font_);
     QString qs = convertText(s, len);
@@ -571,7 +564,7 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len,
     {
         // We can't just add the individual character widths together because
         // of kerning.
-        int width = fm.width(qs.left(i_char + 1));
+        XYPOSITION width = fm.width(qs.left(i_char + 1));
 
         if (unicodeMode)
         {
@@ -582,27 +575,29 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len,
                 positions[i_byte++] = width;
         }
         else
+        {
             positions[i_byte++] = width;
+        }
     }
 }
 
-int SurfaceImpl::WidthText(Font &font_, const char *s, int len)
+XYPOSITION SurfaceImpl::WidthText(Font &font_, const char *s, int len)
 {
     return metrics(font_).width(convertText(s, len));
 
 }
 
-int SurfaceImpl::WidthChar(Font &font_, char ch)
+XYPOSITION SurfaceImpl::WidthChar(Font &font_, char ch)
 {
     return metrics(font_).width(ch);
 }
 
-int SurfaceImpl::Ascent(Font &font_)
+XYPOSITION SurfaceImpl::Ascent(Font &font_)
 {
     return metrics(font_).ascent();
 }
 
-int SurfaceImpl::Descent(Font &font_)
+XYPOSITION SurfaceImpl::Descent(Font &font_)
 {
     // Qt doesn't include the baseline in the descent, so add it.  Note that
     // a descent from Qt4 always seems to be 2 pixels larger (irrespective of
@@ -612,18 +607,27 @@ int SurfaceImpl::Descent(Font &font_)
     return metrics(font_).descent() + 1;
 }
 
-int SurfaceImpl::ExternalLeading(Font &font_)
+XYPOSITION SurfaceImpl::ExternalLeading(Font &font_)
 {
     // Scintilla doesn't use this at the moment, which is good because Qt4 can
     // return a negative value.
     return metrics(font_).leading();
 }
 
-int SurfaceImpl::Height(Font &font_)
+XYPOSITION SurfaceImpl::Height(Font &font_)
 {
     return metrics(font_).height();
 }
 
+XYPOSITION SurfaceImpl::AverageCharWidth(Font &font_)
+{
+#if QT_VERSION >= 0x040200
+    return metrics(font_).averageCharWidth();
+#else
+    return WidthChar(font_, 'n');
+#endif
+}
+
 void SurfaceImpl::SetClip(PRectangle rc)
 {
     Q_ASSERT(painter);
@@ -666,7 +670,7 @@ QString SurfaceImpl::convertText(const char *s, int len)
 }
 
 // Convert a Scintilla colour and alpha component to a Qt QRgb.
-QRgb SurfaceImpl::convertQRgb(const ColourAllocated &col, unsigned alpha)
+QRgb SurfaceImpl::convertQRgb(const ColourDesired &col, unsigned alpha)
 {
     long c = col.AsLong();
 
@@ -678,7 +682,7 @@ QRgb SurfaceImpl::convertQRgb(const ColourAllocated &col, unsigned alpha)
 }
 
 // Convert a Scintilla colour, and alpha component, to a Qt QColor.
-QColor SurfaceImpl::convertQColor(const ColourAllocated &col, unsigned alpha)
+QColor SurfaceImpl::convertQColor(const ColourDesired &col, unsigned alpha)
 {
     QColor c;
     c.setRgb(convertQRgb(col, alpha));
@@ -882,7 +886,7 @@ public:
     virtual Function FindFunction(const char *name)
     {
         if (m)
-            return m->resolve(name);
+            return (Function)m->resolve(name);
 
         return 0;
     }
